//
//  Display.cpp
//  c_gtk_image
//
//  Created by Arnaud Blanchard on 12/11/2020.
//
#include "Display.hpp"

using namespace std;

static GtkToggleToolButton *blgtk_add_toggle_tool_button(GtkWidget *toolbar, char const *label, char const *icon_name, GCallback callback, void *user_data)
{
    GtkToggleToolButton *toggle_tool_button;
    
    toggle_tool_button=GTK_TOGGLE_TOOL_BUTTON(gtk_toggle_tool_button_new());
    gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(toggle_tool_button), icon_name);
    gtk_tool_button_set_label(GTK_TOOL_BUTTON(toggle_tool_button), label);
    gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(toggle_tool_button));
    if (callback) g_signal_connect(G_OBJECT(toggle_tool_button), "clicked", callback, user_data);
    return toggle_tool_button;
}

static GtkWidget *blgtk_add_tool_button(GtkWidget *toolbar, gchar const *label, gchar const *icon_name, GCallback callback, gpointer user_data)
{
    GtkWidget *tool_button;
    
    tool_button=GTK_WIDGET(gtk_tool_button_new(gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR), label));
    gtk_container_add(GTK_CONTAINER(toolbar), tool_button);
    if (callback) g_signal_connect(G_OBJECT(tool_button), "clicked", callback, user_data);
    return tool_button;
}

static void toggle_fullscreen(GtkWidget *widget, GdkEventWindowState *event, Display *display){
    
    if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN ){
        gtk_widget_hide(display->toolbar);
        gtk_widget_hide(display->general_statusbar);
    }
    else{
        gtk_widget_show(display->toolbar);
        gtk_widget_show(display->general_statusbar);
    }
}

static void zoom_original_cb(GtkToolButton *, Display *display){
    display->image->zoom_original();
}

static void zoom_out_cb(GtkToolButton *toolbutton, Display *display){
    display->image->zoom(2);
}

static void zoom_in_cb(GtkToolButton *toolbutton, Display *display){
    display->image->zoom(0.5);
}

static void false_color_cb(GtkToolButton *toolbutton, struct  update_cb *){
    GtkWidget *label, *legend_box, *legend_image;
    char text[NAME_MAX+1];
    int i;
    /*
     
     if (gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(toolbutton))){
     color_map=false_colors;
     legend_surface=cairo_image_surface_create_for_data ((uchar*)false_colors, CAIRO_FORMAT_RGB24, 256, 1, 256*4);
     legend=gtk_box_new(GTK_ORIENTATION_VERTICAL, 1);
     legend_image = gtk_image_new_from_surface(legend_surface);
     legend_box=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 1);
     
     FOR(i, 7)
     {
     SPRINTF(text, "%d", (i+1)*256/8);
     label = gtk_label_new(text);
     gtk_box_pack_start(GTK_BOX(legend_box), label, TRUE, TRUE, 0);
     }
     gtk_container_add(GTK_CONTAINER(legend), legend_image);
     gtk_container_add(GTK_CONTAINER(legend), legend_box);
     gtk_paned_add2(GTK_PANED(paned),legend );
     gtk_widget_show_all(legend);
     }
     else{
     color_map=gray_colors;
     if (legend){
     gtk_container_remove(GTK_CONTAINER(paned), legend);
     gtk_widget_destroy(legend);
     legend=NULL;
     }
     }*/
}

static  gboolean frame_rate(gpointer user_data){
    auto display = reinterpret_cast<Display*>(user_data);
    display->framerate();
    return G_SOURCE_CONTINUE;
}

Display::Display(blc_array const &array):blc_array(array){
    uint32_t type_string,format_string;
    char label_text[NAME_MAX + 1];
    
    switch (format){
        case 'Y800':switch (type){
            case 'UIN8':image=make_unique<UIN8_Y800_image>(*this); //*this because it is a copy constructor of blc_array
                break;
            }
            break;
        case 'yuv2':case 'UYVY':
            switch (type){
          //      case 'UIN8':image=make_unique<UIN8_UYVY_display>(*this);
                    break;
            }
            break;
        default:
            EXIT_ON_ERROR("Unknow image type or format");
    }
    
    widget=gtk_box_new(GTK_ORIENTATION_VERTICAL, 3);
    toolbar = gtk_toolbar_new();
    general_statusbar=gtk_statusbar_new();
    framerate_statusbar=gtk_statusbar_new();
    pointer_statusbar=gtk_statusbar_new();
    
    gtk_container_add(GTK_CONTAINER(general_statusbar), framerate_statusbar);
    gtk_container_add(GTK_CONTAINER(general_statusbar), pointer_statusbar);
    gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
    
    g_timeout_add(1000, frame_rate, this);
    
    SPRINTF(label_text, "%.4s, %.4s, %zxx%zu", UINT32_TO_STRING(type_string, type), UINT32_TO_STRING(format_string, format), image->width, image->height);
    gtk_statusbar_push(GTK_STATUSBAR(general_statusbar), 0, label_text);

    blgtk_add_tool_button(toolbar, "zoom\n1:1", "zoom-original", G_CALLBACK(zoom_original_cb), this);
    blgtk_add_tool_button(toolbar, "zoom\nx0.5", "zoom-out", G_CALLBACK(zoom_out_cb), this);
    blgtk_add_tool_button(toolbar, "zoom\nx2", "zoom-in", G_CALLBACK(zoom_in_cb),  this);
    fit_button=GTK_WIDGET(blgtk_add_toggle_tool_button(toolbar, "zoom\nfit", "zoom-fit-best", nullptr, nullptr));
    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(fit_button), TRUE);
    //    blgtk_add_toggle_tool_button(toolbar, "cursor\nvalues", "system-search", NULL, NULL);/
 //   histogram_button=GTK_WIDGET(blgtk_add_toggle_tool_button(toolbar, "histogram", "x-office-presentation", G_CALLBACK(histogram_cb), this));
    
// GtkWidget *paned=gtk_paned_new(GTK_ORIENTATION_VERTICAL);
//    gtk_paned_set_wide_handle (GTK_PANED(paned), 1);
    
 //   gtk_paned_add1(GTK_PANED(paned), image->widget);
  //  gtk_paned_add2(GTK_PANED(paned), gtk_label_new("bottom"));

    gtk_container_add(GTK_CONTAINER(widget), toolbar);
    gtk_container_add(GTK_CONTAINER(widget), image->widget);
    //gtk_container_add(GTK_CONTAINER(widget), paned);
    gtk_container_add(GTK_CONTAINER(widget), general_statusbar);
    
//    image->zoom_original();
    gtk_widget_show_all(widget);

  //  g_signal_connect(G_OBJECT(widget), "size-allocate", G_CALLBACK(Display::resize_cb), this);

   /*  gtk_widget_set_vexpand(image, TRUE);
     gtk_widget_set_hexpand(image, TRUE);*/
    
    /*g_signal_connect(window, "window-state-event", G_CALLBACK (toggle_fullscreen), this);
    if (fullscreen_option) gtk_window_fullscreen(GTK_WINDOW(window));*/
}

Display::~Display(){
    data=nullptr;
}

void Display::update(){
    if (image) {
        image->update();
        gtk_statusbar_push(GTK_STATUSBAR(pointer_statusbar), 0, image->pointer_info.data());
    }
    
    if (histogram) {
        histogram->update();
        gtk_statusbar_push(GTK_STATUSBAR(pointer_statusbar), 0, histogram->pointer_info.data());
    }
    iterations_nb++;
}

void Display::histogram_cb(){
    
}

void Display::framerate(){
    char text[64];
    
    SPRINTF(text, "%dHz", iterations_nb);
    gtk_statusbar_push(GTK_STATUSBAR(framerate_statusbar), 0, text);
    iterations_nb=0;
}
